local Me=CreateFrame("Frame","vZifnabsTrader")
vZifnabs.mod.trader=Me
local mod=vZifnabs.mod
local pfx="vZifnabs_Trader"
local L={}
local db=nil
local originalTooltipAddMoney
local sellprice
local initialized


-- -----------------------------------------------------------------
-- INITIALIZE
-- -----------------------------------------------------------------
function Me:Initialize()
	db,L=mod.core:InitStart("trader")
	if not db.trader then db.trader={item={}} end
	
	originalTooltipAddMoney = vZifnabs_Trader_Tooltip:GetScript("OnTooltipAddMoney");
	vZifnabs_Trader_Tooltip:SetScript("OnTooltipAddMoney", Me.Tooltip_AddMoney);
	initialized=true
end
	
function Me:Activate()		
	mod.core:RegisterEvent("AUCTION_HOUSE_SHOW","trader",Me.AUCTION_HOUSE_SHOW)
	mod.core:RegisterEvent("AUCTION_HOUSE_CLOSE","trader",Me.AUCTION_HOUSE_CLOSE)
	mod.core:RegisterEvent("NEW_AUCTION_UPDATE","trader",Me.NEW_AUCTION_UPDATE)
	mod.core:RegisterEvent("MERCHANT_SHOW","trader",Me.MERCHANT_SHOW)
end

function Me:Deactivate()
	mod.core:DeRegisterEvent("AUCTION_HOUSE_SHOW","trader")
	mod.core:DeRegisterEvent("AUCTION_HOUSE_CLOSE","trader")
	mod.core:DeRegisterEvent("NEW_AUCTION_UPDATE","trader")
	mod.core:DeRegisterEvent("MERCHANT_SHOW","trader")
end


-- -----------------------------------------------------------------
-- TOOLTIP ADD MONEY
-- -----------------------------------------------------------------
function Me:Tooltip_AddMoney ()
	originalTooltipAddMoney();
	sellprice=arg1
end

-- -----------------------------------------------------------------
-- TOOLTIP ITEM CHANGED
-- -----------------------------------------------------------------
function Me:Tooltip_Item_Changed()
	if initialized then
		local item=getglobal("GameTooltipTextLeft1"):GetText()
		local price=nil
		local ahinfo=nil
		if item and db.trader.item[item] then price=db.trader.item[item].sell
			if db.trader.item[item].ah then
				local ah=db.trader.item[item].ah
				ahinfo=ah.count.."x AH: "..Me:Price(ah.min).." min, "..Me:Price(Math.floor(ah.sum/ah.count)).." avg"
			end
		end
		if price then GameTooltip:AddLine(mod.core:Color(L.msg.sellprice,"ccee00").." "..Me:Price(price),1,1,1) end
		if ahinfo then GameTooltip:AddLine(ahinfo,1,1,1) end
		GameTooltip:Show()
	end
end

-- -----------------------------------------------------------------
-- MERCHANT SHOW
-- -----------------------------------------------------------------
function Me:MERCHANT_SHOW()
    for bag=0,NUM_BAG_FRAMES do
        for slot=1,GetContainerNumSlots(bag) do
			local item = GetContainerItemLink(bag, slot);
            if item then
				local link,_,name = string.find(item, "^.*%[(.*)%].*$");
				if link then item=name end
				sellprice=0
                vZifnabs_Trader_Tooltip:SetBagItem(bag, slot);
				local what,count = GetContainerItemInfo(bag, slot)
				if count and count>0 and sellprice then sellprice=sellprice/count end
				if not db.trader.item[item] then db.trader.item[item]={} end
				db.trader.item[item].sell=sellprice
            end
        end
    end
end

-- -----------------------------------------------------------------
-- AUCTION HOUSE SHOW
-- -----------------------------------------------------------------
function Me:AUCTION_HOUSE_SHOW()
	mod.core:Print("AH opened")
end

-- -----------------------------------------------------------------
-- AUCTION HOUSE CLOSE
-- -----------------------------------------------------------------
function Me:AUCTION_HOUSE_CLOSE()
	mod.core:Print("AH closed")
end

-- -----------------------------------------------------------------
-- NEW AUCTION UPDATE
-- -----------------------------------------------------------------
function Me:NEW_AUCTION_UPDATE()
-- db.trader.item[item].ah={count=?,sum=?,min=?,max=?}
	local name, texture, count, quality, canUse, price = GetAuctionSellItemInfo();
	if name then
		if price and not db.trader.item[name] then db.trader.item[name]={sell=price} end
		mod.core:Print("AH auction for "..count.." x "..name.." stack price: "..Me:Price(price))
	end
end

-- -----------------------------------------------------------------
-- PRICE (format copper to gg.ss.cc)
-- -----------------------------------------------------------------
function Me:Price(copper)
	local price=L.keyword.unknown
	if copper then 
		local gold=math.floor(copper/10000)
		local silver=math.floor((copper-10000*gold)/100)
		copper=copper-10000*gold-100*silver
		price=mod.core:Color(copper,"ee6600")
		if silver>0 then price=mod.core:Color(silver,"dddddd")..","..price end
		if gold>0 then price=mod.core:Color(gold,"ddbb00")..","..price end
	end
	return price
end