local vZifnabsRelease="r190316a" 

local Me=CreateFrame("Frame","vZifnabsCore")
vZifnabs={mod={core=Me},release=vZifnabsRelease,initialization={}}

-- Database default definitions
vZifnabsDB={frames={}}

-- Slash Commands
SLASH_VZIFNABS1 = "/z"
SlashCmdList["VZIFNABS"]=function (cmd) Me:SlashCommand(cmd) end

-- Local shortcuts to globals
local db=nil
local mod=vZifnabs.mod
local L={}

-- local work variables
local buttonlist={}
local eventlist={}
local unknownevents={}
local lasttime=0
local tick={time=0,last=0,robin=0}
local updates=0
local fps={}
local widget={}
local config={mod={}}

BINDING_HEADER_VZIFNABS = "vZifnabs"

-- -----------------------------------------
-- INITIALIZE
-- -----------------------------------------
function Me:Initialize()
	db=vZifnabsDB
	L=vZifnabs.L
	vZifnabs.release=vZifnabsRelease
	if not db.core then db.core={FPS={min=20,max=40,deactivated}} end
	if not db.core.deactivated then db.core.deactivated={} end -- new after r190310b
	fps=vZifnabsDB.core.FPS

	for frame,pos in pairs(vZifnabsDB.frames) do
		if getglobal(frame) then getglobal(frame):SetPoint(pos.point,"UIParent",pos.point,pos.x,pos.y) end
	end

	vZifnabs.mod.config:Initialize() -- has to be first as others have to register to it
	Me:InitializeConfig()
	for name,mod in pairs(vZifnabs.mod) do
		if name~="core" and name~="config" and mod.Initialize then 
			mod:Initialize() 
			if L.mod[name] then Me:SetModState(name) end
		end
	end
	Me:Print(Me:Repl(L.msg.initialized,"<release>",vZifnabsRelease))
	db.release=vZifnabsRelease
end
function Me:InitStart(mod) return vZifnabsDB,vZifnabs.L end

function Me:SetModState(modname)
	if db.core.deactivated[modname] then 
		Me:Print(Me:Repl(L.msg.moddeactivate,"<mod>",L.mod[modname]))
		mod[modname]:Deactivate()
	else 
		Me:Print(Me:Repl(L.msg.modactivate,"<mod>",L.mod[modname])) 
		mod[modname]:Activate()
	end
end

-- -----------------------------------------
-- REGISTER EVENT
-- -----------------------------------------
function Me:RegisterEvent(event,mod,method)
	if mod==nil or method==nil then Me:Print("Error on Register Event for "..(event or "nil").." from "..(mod or "nil")) end
	if not eventlist[event] then 
		eventlist[event]={} 
		getglobal("vZifnabs_EventFrame"):RegisterEvent(event)
	end
	if event=="TICK" then 
		eventlist.TICK[1+getn(eventlist.TICK)]={mod=mod,method=method,active=true}
	else	
		eventlist[event][mod]={method=method,active=true}
	end
end

function Me:DeRegisterEvent(event,mod)
	if eventlist[event] and eventlist[event][mod] then eventlist[event][mod]=nil end
	local count=0
	for emod,data in pairs(eventlist[event] or {}) do count=count+1 end
	if event~="TICK" and count==0 then getglobal("vZifnabs_EventFrame"):UnregisterEvent(event) end
end

-- -----------------------------------------
-- ON EVENT
-- -----------------------------------------
function Me:OnEvent(event,arg1,arg2,arg3,arg4)
	if event=="VARIABLES_LOADED" then Me:Initialize()
	elseif eventlist[event] then
		for mod,info in pairs(eventlist[event]) do
			if info.active then 
				if not info.method then Me:Print("INTERNAL ERROR: No method for "..event.." in "..mod)
				else
					info.method(nil,event,arg1,arg2,arg3,arg4) -- "nil" because method call via mod.method instead of mod:method
				end
			end 
		end
	elseif not unknownevents[event] then 
		Me:Print(Me:Repl(L.msg.unknownevent,"<event>",event))
		unknownevents[event]=true
	end
end

-- -----------------------------------------
-- ON UPDATE
-- -----------------------------------------
function Me:OnUpdate(elapsed)
	local newtime=time()
	updates=updates+1
	tick.time=tick.time+elapsed
	if tick.time>tick.last+0.1 or tick.robin>0 then
		if tick.robin==0 then tick.last=tick.time end
		tick.robin=tick.robin+1
		if tick.robin> getn(eventlist.TICK) then 
			tick.robin=0
		else 
			eventlist.TICK[tick.robin].method(nil,"TICK",tick.time)
		end
	end
	if newtime>lasttime then
		-- Show Clock
		getglobal("vZifnabs_Control_Clock"):SetText(date("%H:%M:%S",newtime))
		
		-- Show FPS
		fps.value=math.floor(updates/(newtime-lasttime))
		local fpstext=getglobal("vZifnabs_Control_FPS")
		local colorrange=(fps.max-fps.min)/2
		local red=min(colorrange,max(0,fps.max-fps.value))/colorrange
		local green=min(colorrange,max(0,fps.value-fps.min))/colorrange
		fpstext:SetText(fps.value)
		fpstext:SetTextColor(red,green,0,1)
		
		Me:OnEvent("CLOCK",newtime)
		
		-- Reset counter
		updates=0
		lasttime=newtime
	end
	Me:OnEvent("ONUPDATE",elapsed)
end

-- -----------------------------------------
-- SLASH COMMAND
-- -----------------------------------------
function Me:SlashCommand(cmd)
	local word=Me:Split(string.lower(cmd)," ")
	if word[1]=="debug" then
		if not word[2] or word[2]=="on" then db.debug=true
		elseif word[2]=="off" then db.debug=false
		else Me:Print(Me:Repl(L.msg.undefparm,"<parm>",word[2] or "nil"))
		end
	elseif word[1]=="remind" then
		word=Me:Split(cmd," ",3)
		vZifnabs.mod.scheduler:Remind(tonumber(word[2]),word[3])
	elseif word[1]=="buffs" then vZifnabs.mod.bufmon:List()
	elseif word[1]=="sound" then PlaySound(word[2])
	elseif word[1]=="rezone" then -- "rezone" for check, "rezone real" for real execution
		local db=vZifnabsDB
		local mod=vZifnabs.mod
		if not db.locator.unknownzones then Me:Print(L.msg.nounknownzones)
		else
			for name,data in pairs(db.locator.unknownzones) do
				if name~="next_nr" then Me:Print(Me:Repl(L.msg.undefzone,{"<nr>","<zone>"},{data.nr,name})) end
			end
		end
		
		local real=0
		if word[2]=="real" then real=1 end
		local unum=1*(word[2+real] or 0)
		local nname=word[3+real]
		local nnum=0
		for w=4+real,99,1 do
			if word[w] then nname=nname+" "+word[w] end
		end
		if nname then
			for name,data in pairs(L.locator.zones) do
				if string.lower(name)==nname then 
					nnum=data.nr
					Me:Print(Me:Repl(L.msg.rezone,{"<oldnum>","<newnum>","<zone>"},{unum,nnum,name}))
				end
			end
			if nnum==0 then Me:Print(Me:Repl(L.msg.zonenotfound,"<zone>",nname)) end
		end
		
		local undef=db.locator.unknownzones
		local seen=db.targets.seen
		local ucount={}
		for name,entry in pairs(seen) do
			for n=1,10,1 do
				if entry.pos[n] then
					local x,y,zone,nr=mod.locator:ParseLoc(entry.pos[n])
					if nr>900 then ucount[nr]=(ucount[nr] or 0)+1 end
					if nr==unum then
						local nx,ny,nzone,npos,nnr=mod.locator:GetLoc(nzone,x/100,y/100)
						if real==1 then seen[name].pos[n]=npos end
						ucount.changed=(ucount.changed or 0)+1
					end
				end
			end
		end
		
		for uzone,count in pairs(ucount) do
			Me:Print(uzone..": "..count)
		end
		
		if undef~=nil and nnum>0 then
			local keep=0
			for zone,info in pairs(undef) do
				if zone~="next_nr" then
					if unum==info.nr then
						Me:Print(Me:Repl(L.msg.erasezone,"<zone>",zone))
						if real==1 then undef[zone]=nil end
					else
						keep=keep+1
					end
				end
			end
			if keep==0 then
				Me:Print(L.msg.erasezoneunknown)
				if real then db.locator.unknownzones=nil end
			end
		end
		if real==1 then Me:Print("REAL") else Me:Print("SIMULATED") end
	else
		vZifnabs.mod.emotor:SlashCommand(cmd)
	end
end

-- -----------------------------------------
-- FRAME POSITION
-- -----------------------------------------
function Me:FramePosition(frame)
	local point,relativeTo,relativePoint,xOffset,yOffset=frame:GetPoint(1)
	vZifnabsDB.frames[frame:GetName()]={point=point,x=xOffset,y=yOffset}
end

-- -----------------------------------------
-- TOOLTIP
-- -----------------------------------------
--<OnEnter>vZifnabsCore:ToolTip(this)</OnEnter><OnLeave>GameTooltip:Hide()</OnLeave>
function Me:ToolTip(widget)
	if widget.tooltipText then
		GameTooltip:SetOwner(widget, "ANCHOR_RIGHT")
		GameTooltip:SetText(widget.tooltipText, 1.0, 0.8, 0.0, 1 )
		GameTooltip:Show();
	end
end

-- -----------------------------------------
-- REGISTER BUTTON
-- -----------------------------------------
function Me:RegisterButton(mod,fnk,def,name,hide)
	if not name then 
		buttonlist.ctlnr=1+(buttonlist.ctlnr or 0)
		name="vZifnabs_Control_"..buttonlist.ctlnr
	end
	local button=getglobal(name)
	if button==nil then Me:Print("ERROR: unexistant button: '"..name.."'")
	else
		buttonlist[name]={mod=mod,fnk=fnk}
		button:SetText(def.text or "?")
		if hide==true then button:Hide() else button:Show() end
	end
	if def.tt then 
		button.tooltipText=def.tt 		
	end
end

-- -----------------------------------------
-- BUTTON
-- -----------------------------------------
function Me:Button(button,mod,fnk)
	if not mod then
		mod=buttonlist[button:GetName()].mod
		fnk=buttonlist[button:GetName()].fnk
	end
	vZifnabs.mod[mod]:Button(button,fnk)
end

-- -----------------------------------------
-- CHECKBOX
-- -----------------------------------------
function Me:Check(checkbox)
	db.core.deactivated[config.mod[checkbox.nr].name]=not checkbox:GetChecked()
	Me:SetModState(config.mod[checkbox.nr].name)
end

-- -----------------------------------------
-- PRINT
-- -----------------------------------------
function Me:PrintInfo(text) Me:Print(Me:Color(text,"88aa44"),nil,true) end
function Me:Print(text,soundname,noprefix)
	if not noprefix then text="|cffddbb00vZifnabs:|r "..Me:Color(text,"ccee44") end
	DEFAULT_CHAT_FRAME:AddMessage(text)
	if soundname~=nil then PlaySound(soundname) end
end

-- -----------------------------------------
-- COLOR
-- -----------------------------------------
function Me:Color(text,color,ambient)
	return "|c"..(ambient or "ff")..color..text.."|r"
end

-- -----------------------------------------
-- REPL
-- -----------------------------------------
function Me:Repl(text,search,repl)
	if not text or not search or not repl then Me:Print("ERROR repl: t='"..(text or "nil").."', s='"..(search or "nil").."', r='"..(repl or "nil").."'") end
	local result=text
	if type(search)~="table" then result=Me:gsub_replacement(result,search,repl)
	else
		for n=1,getn(search),1 do
			result=Me:gsub_replacement(result,search[n],repl[n])
		end
	end
	return result
end

function Me:gsub_replacement(text,search,replacement)
	if not text or not search or not replacement then 
		Me:Print("ERROR gsub: t='"..(text or "nil").."', s='"..(search or "nil").."', r='"..(replacement or "nil").."'") 
		text=text or "nil"
		search=search or "nil"
		replacement=replacement or "nil"
	end
	local searchlen=strlen(search)
	local i=0
	while text and i<string.len(text) do
		if string.sub(text,i,i+searchlen-1)~=search then i=i+1
		else
			text=string.sub(text,1,i-1)..replacement..string.sub(text,i+searchlen)
			i=i+strlen(replacement)
		end
	end
	return text
end


-- -----------------------------------------
-- TRIM
-- -----------------------------------------
function Me:Trim(text)
	while text and string.len(text)>0 and string.sub(text,1,1)==" " do 
		text=string.sub(text,2) 
	end
	while text and string.len(text)>0 and string.sub(text,-1,-1)==" " do 
		text=string.sub(text,1,-2) 
	end
	return text
end

-- -----------------------------------------
-- SPLIT
-- -----------------------------------------
function Me:Split(text,separator,maxparts)
	local part={}
	local pn=0
	while text and text~="" and (not maxparts or pn<maxparts) do
		if separator==" " then text=Me:Trim(text) end
		local ix=string.find(text..separator,separator)
		pn=pn+1
		if maxparts and pn==maxparts then part[pn]=text
		else
			part[pn]=string.sub(text,1,ix-1)
			text=string.sub(text,ix+string.len(separator))
		end
	end
	return part
end

-- -----------------------------------------------------------------
-- CONFIGURATION INITIALIZATION
-- -----------------------------------------------------------------
function Me:InitializeConfig()
	vZifnabs.mod.config:Register(Me,L.button.core.config,"vZifnabs_Core_Config")
	local prefix="vZifnabs_Core_Config"
	widget.modtitle=getglobal(prefix.."_ModTitle")
	widget.modtitle:SetText(L.core.modtitle)
	widget.mod={}
	for nr=1,8,1 do 
		widget.mod[nr]={check=getglobal(prefix.."_UseMod_"..nr),name=getglobal(prefix.."_Modname_"..nr)}
		widget.mod[nr].check:Hide()
		widget.mod[nr].check.nr=nr
		widget.mod[nr].name:SetText("")
	end
	local nr=0
	for modname,object in pairs(vZifnabs.mod) do
		if L.mod[modname] then
			nr=nr+1
			config.mod[nr]={name=modname,mod=object}
		end
	end
end

-- -----------------------------------------------------------------
-- CONFIGURATION START
-- -----------------------------------------------------------------
function Me:ConfigStart()
	for nr=1,8,1 do
		if config.mod[nr] then 
			widget.mod[nr].check:Show()
			widget.mod[nr].check:SetChecked(db.core.deactivated[config.mod[nr].name]~=true)
			widget.mod[nr].name:SetText(L.mod[config.mod[nr].name])
		end
	end
end


-- -----------------------------------------------------------------
-- CONFIGURATION SAVE, DROP AND STOP
-- nothing to do because changes activated immediately
-- -----------------------------------------------------------------
function Me:ConfigSave() end
function Me:ConfigDrop() Me:ConfigStart() end
function Me:ConfigStop() end
